@@ -38,6 +38,7 @@ def model_and_rows(model, rows):
 
 def build_as_view_wrapper(target):
     from iommi.path import decode_path_components  # avoid circular import
+
     if not target.is_refine_done and getattr(settings, 'IOMMI_REFINE_DONE_OPTIMIZATION', True):
         target = target.refine_done()
 
@@ -81,7 +82,7 @@ def get_display_name(part):
 
 
 def get_wrapped_view(view):
-    if hasattr(view, '__iommi_target__'):
+    while hasattr(view, '__iommi_target__'):
         view = view.__iommi_target__
 
     while hasattr(view, '__wrapped__'):
