@@ -148,7 +148,7 @@ def many_to_many_factory_read_from_instance(field, instance):
 
 
 def many_to_many_factory_write_to_instance(field, instance, value):
-    getattr_path(instance, field.attr).set(value)
+    getattr_path(instance, field.attr).set(value or [])
 
 
 _field_factory_by_field_type = {}
@@ -181,51 +181,53 @@ def find_unique_prefixes(attributes):
 
 
 def create_or_edit_object__post_handler(*, form, is_create, **_):
+    if not form.is_valid():
+        return
+
     if is_create:
         assert form.instance is None
-        form.instance = evaluate(form.extra.new_instance, **form.iommi_evaluate_parameters())
+        form.instance = form.invoke_callback(form.extra.new_instance)
 
         # `own_evaluate_parameters` can't be used here because it is calculated too early when the instance doesn't exist yet. So instead we have to insert it manually when we get it.
         # noinspection PyProtectedMember
         form._evaluate_parameters['instance'] = form.instance
 
-        for field in values(
-            form.fields
-        ):  # two phase save for creation in django, have to save main object before related stuff
+        for field in values(form.fields):
+            # two phase save for creation in django, have to save main object before related stuff
             if not field.extra.get('django_related_field', False):
                 form.apply_field(field=field, instance=form.instance)
 
-    with validation_errors_reported_on(form):
-        form.instance.validate_unique()
+        with validation_errors_reported_on(form):
+            form.instance.validate_unique()
+        if not form.is_valid():
+            return
 
-    if not form.is_valid():
-        return
-
-    if is_create:  # two phase save for creation in django...
-        form.extra.pre_save_all_but_related_fields(**form.iommi_evaluate_parameters())
+        # two phase save for creation in django...
+        form.invoke_callback(form.extra.pre_save_all_but_related_fields)
         form.instance.save()
-        form.extra.on_save_all_but_related_fields(**form.iommi_evaluate_parameters())
+        form.invoke_callback(form.extra.on_save_all_but_related_fields)
 
     form.apply(form.instance)
 
     if not is_create:
         with validation_errors_reported_on(form):
             form.instance.validate_unique()
+        if not form.is_valid():
+            return
 
-    if form.is_valid():
-        attributes = filter(None, [f.attr for f in form.fields.values()])
+    attributes = filter(None, [f.attr for f in form.fields.values()])
 
-        form.extra.pre_save(**form.iommi_evaluate_parameters())
-        for prefix in find_unique_prefixes(attributes):
-            model_object = form.instance
-            if prefix:  # Might be ''
-                model_object = getattr_path(model_object, prefix)
-            model_object.save()
-        form.extra.on_save(**form.iommi_evaluate_parameters())
+    form.invoke_callback(form.extra.pre_save)
+    for prefix in find_unique_prefixes(attributes):
+        model_object = form.instance
+        if prefix:  # Might be ''
+            model_object = getattr_path(model_object, prefix)
+        model_object.save()
+    form.invoke_callback(form.extra.on_save)
 
-        return create_or_edit_object_redirect(
-            is_create, form.extra.redirect_to, form.get_request(), form.extra.redirect, form
-        )
+    return create_or_edit_object_redirect(
+        is_create, form.extra.redirect_to, form.get_request(), form.extra.redirect, form
+    )
 
 
 def default_endpoints__config(field: 'Field', **_) -> dict:
@@ -259,7 +261,7 @@ def choice_is_valid(field, parsed_data, **_):
 
 def choice_parse(form, field, string_value):
     for choice in field.choices:
-        if string_value == field.choice_id_formatter(choice=choice, **field.iommi_evaluate_parameters()):
+        if string_value == field.invoke_callback(field.choice_id_formatter, choice=choice):
             return choice
 
     if string_value in [None, '']:
@@ -303,10 +305,9 @@ def choice_queryset__endpoint_handler(*, form, field, value, page_size=40, **_):
 def choice_queryset__extra__model_from_choices(form, field, choices):
     def traverse():
         for choice in choices:
-            option = field._build_option(choice)
             yield Struct(
-                id=option[1],
-                text=option[2],
+                id=field.invoke_callback(field.choice_id_formatter, choice=choice),
+                text=field.invoke_callback(field.choice_display_name_formatter, choice=choice),
             )
 
     return list(traverse())
@@ -555,8 +556,12 @@ class Field(Part, Tag):
         input__attrs__id=default_input_id,
         input__attrs__name=lambda field, **_: field.iommi_path,
         input__extra__placeholder='',
-        non_editable_input__call_target=Fragment,
-        non_editable_input__attrs__disabled=True,
+        non_editable_input=dict(
+            call_target=Fragment,
+            children__text=lambda fragment, field, **_: None if fragment.tag == 'input' else field.rendered_value,
+            attrs__value=lambda fragment, field, **_: field.rendered_value if fragment.tag == 'input' else None,
+            attrs__disabled=lambda fragment, field, **_: True if fragment.tag == 'input' else None,
+        ),
         initial=MISSING,
         choice_to_optgroup=None,
         choice_id_formatter=lambda choice, **_: '%s' % choice,
@@ -602,7 +607,9 @@ class Field(Part, Tag):
 
     def on_refine_done(self):
         if 'choice' in getattr(self, 'iommi_shortcut_stack', []):
-            assert self.iommi_namespace.get('choices') is not None, 'To use Field.choice, you must pass the choices list'
+            assert (
+                self.iommi_namespace.get('choices') is not None
+            ), 'To use Field.choice, you must pass the choices list'
 
         model_field = self.model_field
         if model_field and model_field.remote_field:
@@ -650,9 +657,9 @@ class Field(Part, Tag):
 
     @staticmethod
     @refinable
-    def render_value(form: 'Form', field: 'Field', value: Any) -> str:
+    def render_value(form: 'Form', field: 'Field', value: Any, **kwargs) -> str:
         if isinstance(value, (list, QuerySet)):
-            return ', '.join(field.render_value(form=form, field=field, value=v) for v in value)
+            return ', '.join(field.render_value(form=form, field=field, value=v, **kwargs) for v in value)
         else:
             return f'{value}' if value is not None else ''
 
@@ -685,6 +692,7 @@ class Field(Part, Tag):
         self.parsed_data = self._parsed_data
         self._errors = set()
         self.choices = evaluate_strict(self.choices, **self.iommi_evaluate_parameters())
+        self.editable = evaluate_strict(self.editable, **self.iommi_evaluate_parameters())
         self.initial = evaluate_strict(self.initial, **self.iommi_evaluate_parameters())
         self._read_initial()
 
@@ -699,19 +707,9 @@ class Field(Part, Tag):
             self._validate()
 
         if self.editable:
-            self.input = self.iommi_namespace.input(_name='input').bind(parent=self)
+            self.input = self.input.bind(parent=self)
         else:
-            if self.iommi_namespace.non_editable_input.get('tag', self.iommi_namespace.input.get('tag')) == 'input':
-                self.input = self.iommi_namespace.input(
-                    attrs__value=self.rendered_value,
-                    attrs__disabled=True,
-                    **self.iommi_namespace.non_editable_input
-                ).bind(parent=self)
-            else:
-                self.input = self.iommi_namespace.input(
-                    children__text=self.rendered_value,
-                    **self.iommi_namespace.non_editable_input
-                ).bind(parent=self)
+            self.input = self.non_editable_input.bind(parent=self)
 
         if self.is_boolean:
             if 'checked' not in self.input.attrs and self.value:
@@ -890,19 +888,15 @@ class Field(Part, Tag):
     def rendered_value(self):
         if self.errors:
             return self.raw_data
-        return self.render_value(form=self.form, field=self, value=self.value)
+        return self.invoke_callback(self.render_value, value=self.value)
 
     def _build_option(self, choice):
         # The legacy structure is `(choice, id, display_name, is_selected)`
         return (
             choice,
-            self.choice_id_formatter(choice=choice, **self.iommi_evaluate_parameters()),
-            self.choice_display_name_formatter(choice=choice, **self.iommi_evaluate_parameters()),
-            (
-                (choice == self.value)
-                if not self.is_list else
-                (self.value is not None and choice in self.value)
-            ),
+            self.invoke_callback(self.choice_id_formatter, choice=choice),
+            self.invoke_callback(self.choice_display_name_formatter, choice=choice),
+            ((choice == self.value) if not self.is_list else (self.value is not None and choice in self.value)),
         )
 
     @property
@@ -939,7 +933,7 @@ class Field(Part, Tag):
             groups.append((current_group_name, current_group))
             for choice_tuple in self.choice_tuples:
                 choice = choice_tuple[0]
-                group_name = self.choice_to_optgroup(choice=choice, **self.iommi_evaluate_parameters())
+                group_name = self.invoke_callback(self.choice_to_optgroup, choice=choice)
                 if current_group_name != group_name:
                     current_group_name = group_name
                     current_group = []
@@ -1094,9 +1088,7 @@ class Field(Part, Tag):
         instance = instance.refine(
             Prio.shortcut,
             choices=(
-                (lambda form, **_: choices.all())
-                if isinstance(choices, QuerySet)
-                else choices
+                (lambda form, **_: choices.all()) if isinstance(choices, QuerySet) else choices
             ),  # clone the QuerySet if needed
         )
         return instance
@@ -1243,17 +1235,16 @@ class Field(Part, Tag):
 
 def is_django_promise_with_string_proxy(redirect_to):
     return (
-            isinstance(redirect_to, Promise)
-            and redirect_to._proxy____kw == {}
-            and len(redirect_to._proxy____args) == 1
-            and isinstance(redirect_to._proxy____args[0], str)
+        isinstance(redirect_to, Promise)
+        and redirect_to._proxy____kw == {}
+        and len(redirect_to._proxy____args) == 1
+        and isinstance(redirect_to._proxy____args[0], str)
     )
 
+
 def create_or_edit_object_redirect(is_create, redirect_to, request, redirect, form):
     assert (
-            redirect_to is None
-            or isinstance(redirect_to, str)
-            or is_django_promise_with_string_proxy(redirect_to)
+        redirect_to is None or isinstance(redirect_to, str) or is_django_promise_with_string_proxy(redirect_to)
     ), 'redirect_to must be a str'
     if redirect_to is None:
         if is_create:
@@ -1269,7 +1260,7 @@ def create_or_edit_object_redirect(is_create, redirect_to, request, redirect, fo
 
 def delete_object__post_handler(form, **_):
     instance = form.instance
-    form.extra.on_delete(**form.iommi_evaluate_parameters())
+    form.invoke_callback(form.extra.on_delete)
     if instance.pk is not None:  # Check if already deleted by the callback
         try:
             instance.delete()
@@ -1284,26 +1275,34 @@ def delete_object__post_handler(form, **_):
 
             # This message must match the one in Django exactly to get translations for free
             # language=html
-            form.add_error(Template("""
-            {% load i18n %}
-            <p>{% blocktrans with escaped_object=object %}Deleting the {{ object_name }} '{{ escaped_object }}' would require deleting the following protected related objects:{% endblocktrans %}</p>
+            form.add_error(
+                Template(
+                    """
+                        {% load i18n %}
+                        <p>{% blocktrans with escaped_object=object %}Deleting the {{ object_name }} '{{ escaped_object }}' would require deleting the following protected related objects:{% endblocktrans %}</p>
 
-            <ul>
-                {% for obj in restricted_objects %}
-                    <li>
-                        {% if obj.get_absolute_url %}
-                            <a href="{{ obj.get_absolute_url }}">{{ obj }}</a>
-                        {% else %}
-                            {{ obj }}
-                        {% endif %}
-                    </li>
-                {% endfor %}
-            </ul>
-            """).render(context=Context(dict(
-                restricted_objects=objects,
-                object=instance,
-                object_name=instance._meta.verbose_name,
-            ))))
+                        <ul>
+                            {% for obj in restricted_objects %}
+                                <li>
+                                    {% if obj.get_absolute_url %}
+                                        <a href="{{ obj.get_absolute_url }}">{{ obj }}</a>
+                                    {% else %}
+                                        {{ obj }}
+                                    {% endif %}
+                                </li>
+                            {% endfor %}
+                        </ul>
+                    """
+                ).render(
+                    context=Context(
+                        dict(
+                            restricted_objects=objects,
+                            object=instance,
+                            object_name=instance._meta.verbose_name,
+                        )
+                    )
+                )
+            )
             return None
 
     return create_or_edit_object_redirect(
@@ -1399,7 +1398,7 @@ class Form(Part):
         # @test
         post_handler(form.bind(request=req('post')))
         # @end
-        """
+    """
 
     actions: Namespace = RefinableMembers()
     actions_template: Union[str, Template] = Refinable()
@@ -1451,7 +1450,9 @@ class Form(Part):
         extra_member_defaults = dict()
         if self.auto:
             auto = FormAutoConfig(**self.auto).refine_done(parent=self)
-            assert not self.model, "You can't use the auto feature and explicitly pass model. Either pass auto__model, or we will set the model for you from auto__instance"
+            assert (
+                not self.model
+            ), "You can't use the auto feature and explicitly pass model. Either pass auto__model, or we will set the model for you from auto__instance"
             if auto.model is None:
                 auto.model = auto.instance.__class__
 
@@ -1490,8 +1491,22 @@ class Form(Part):
 
         assert isinstance(self.fields, dict)
 
-        refine_done_members(self, name='actions', members_from_namespace=self.actions, cls=self.get_meta().action_class, members_cls=Actions)
-        refine_done_members(self, name='fields', members_from_namespace=self.fields, members_from_declared=self.get_declared('_fields_dict'), members_from_auto=fields_from_auto, cls=self.get_meta().member_class, extra_member_defaults=extra_member_defaults,)
+        refine_done_members(
+            self,
+            name='actions',
+            members_from_namespace=self.actions,
+            cls=self.get_meta().action_class,
+            members_cls=Actions,
+        )
+        refine_done_members(
+            self,
+            name='fields',
+            members_from_namespace=self.fields,
+            members_from_declared=self.get_declared('_fields_dict'),
+            members_from_auto=fields_from_auto,
+            cls=self.get_meta().member_class,
+            extra_member_defaults=extra_member_defaults,
+        )
 
         super(Form, self).on_refine_done()
 
@@ -1539,7 +1554,6 @@ class Form(Part):
 
         self.parts = self.fields
         self.fields = self.all_fields
-        del self.all_fields
 
         self.errors = Errors(parent=self, **self.errors)
 
@@ -1594,7 +1608,10 @@ class Form(Part):
         return model, fields
 
     def is_target(self):
-        this_form_is_target = any(action.is_target() for action in values(self.actions))
+        self.iommi_path  # Make sure path tree is calculated top down.
+        declared_members = object.__getattribute__(self.actions, '_declared_members')
+        this_form_is_target = any(f'-{action.iommi_path}' in self._request_data for action in values(declared_members))
+
         if this_form_is_target:
             return True
         if self.parent_form is not None:
@@ -1618,7 +1635,7 @@ class Form(Part):
             self._valid = True
         for field in values(self.fields):
             with validation_errors_reported_on(field):
-                field.post_validation(**field.iommi_evaluate_parameters())
+                field.invoke_callback(field.post_validation)
 
         for nested_form in values(self.nested_forms):
             # At this point the nested forms are already validated, because
@@ -1628,7 +1645,7 @@ class Form(Part):
 
         if self.mode is FULL_FORM_FROM_REQUEST:
             with validation_errors_reported_on(self):
-                self.post_validation(**self.iommi_evaluate_parameters())
+                self.invoke_callback(self.post_validation)
 
         return self
 
