@@ -235,7 +235,7 @@ def ordered_by_on_list(objects, order_field, is_desc=False):
 
 
 def yes_no_formatter(value, **_):
-    """ Handle True/False from Django model and 1/0 from raw sql """
+    """Handle True/False from Django model and 1/0 from raw sql"""
     if value is None:
         return ''
     if value == 1:  # boolean True is equal to 1
@@ -418,7 +418,9 @@ class Column(Part):
 
     def on_refine_done(self):
         if 'choice' in getattr(self, 'iommi_shortcut_stack', []):
-            assert self.iommi_namespace.get('choices') is not None, 'To use Column.choice, you must pass the choices list'
+            assert (
+                self.iommi_namespace.get('choices') is not None
+            ), 'To use Column.choice, you must pass the choices list'
 
         self.header = HeaderColumnConfig(**self.header).refine_done(parent=self)
         self.is_sorting: bool = None
@@ -1101,12 +1103,12 @@ def bulk__post_handler(table, form, **_):
                 getattr(obj, attr).set(value)
             obj.save()
 
-    response = table.post_bulk_edit(
+    response = table.invoke_callback(
+        table.post_bulk_edit,
         pks=pks,
         queryset=queryset,
         updates=updates,
         m2m_updates=m2m_updates,
-        **table.iommi_evaluate_parameters(),
     )
 
     if response is not None:
@@ -1478,7 +1480,9 @@ class Table(Part, Tag):
         action_class = Action
         page_class = Page
         cells_class = Cells
-        endpoints__tbody__func = lambda table, **_: {'html': table.__html__(template='iommi/table/table_container.html')}
+        endpoints__tbody__func = lambda table, **_: {
+            'html': table.__html__(template='iommi/table/table_container.html')
+        }
         endpoints__csv__func = endpoint__csv
 
         attrs = Namespace(
@@ -1621,8 +1625,22 @@ class Table(Part, Tag):
         self.sorted_and_filtered_rows = None
         self.visible_rows = None
 
-        refine_done_members(self, name='actions', members_from_namespace=self.actions, cls=self.get_meta().action_class, members_cls=Actions)
-        refine_done_members(self, name='columns', members_from_namespace=self.columns, members_from_declared=self.get_declared('_columns_dict'), members_from_auto=columns_from_auto, cls=self.get_meta().member_class, extra_member_defaults=extra_column_defaults,)
+        refine_done_members(
+            self,
+            name='actions',
+            members_from_namespace=self.actions,
+            cls=self.get_meta().action_class,
+            members_cls=Actions,
+        )
+        refine_done_members(
+            self,
+            name='columns',
+            members_from_namespace=self.columns,
+            members_from_declared=self.get_declared('_columns_dict'),
+            members_from_auto=columns_from_auto,
+            cls=self.get_meta().member_class,
+            extra_member_defaults=extra_column_defaults,
+        )
 
         if not self.sortable:
             for column in values(self.iommi_namespace.columns):
@@ -1683,13 +1701,15 @@ class Table(Part, Tag):
 
                 filters[name] = filter()
 
-            self.query = self.get_meta().query_class(**setdefaults_path(
-                Namespace(),
-                query_args,
-                filters=filters,
-                _name='query',
-                model=self.model,
-            ))
+            self.query = self.get_meta().query_class(
+                **setdefaults_path(
+                    Namespace(),
+                    query_args,
+                    filters=filters,
+                    _name='query',
+                    model=self.model,
+                )
+            )
 
             declared_filters = self.query.iommi_namespace.filters
             self.query = self.query.refine(Prio.table_defaults, filters=declared_filters)
@@ -1726,24 +1746,29 @@ class Table(Part, Tag):
             add_hidden_all_pks_field(declared_bulk_fields)
 
             # x.bulk.include can be a callable here. We treat that as truthy on purpose.
-            if any(x.bulk.include for x in values(self.iommi_namespace.columns)) or 'actions' in self.iommi_namespace.bulk:
-                self.bulk = form_class(**setdefaults_path(
-                    Namespace(),
-                    bulk_args,
-                    _name='bulk',
-                    model=self.model,
-                    actions__submit=dict(
-                        post_handler=bulk__post_handler,
-                        display_name=gettext_lazy('Bulk change'),
-                        include=lambda table, **_: any(c.bulk.include for c in values(table.columns)),
-                    ),
-                    actions__delete=dict(
-                        call_target__attribute='delete',
-                        post_handler=bulk_delete__post_handler,
-                        display_name=gettext_lazy('Bulk delete'),
-                        include=False,
-                    ),
-                )).refine(
+            if (
+                any(x.bulk.include for x in values(self.iommi_namespace.columns))
+                or 'actions' in self.iommi_namespace.bulk
+            ):
+                self.bulk = form_class(
+                    **setdefaults_path(
+                        Namespace(),
+                        bulk_args,
+                        _name='bulk',
+                        model=self.model,
+                        actions__submit=dict(
+                            post_handler=bulk__post_handler,
+                            display_name=gettext_lazy('Bulk change'),
+                            include=lambda table, **_: any(c.bulk.include for c in values(table.columns)),
+                        ),
+                        actions__delete=dict(
+                            call_target__attribute='delete',
+                            post_handler=bulk_delete__post_handler,
+                            display_name=gettext_lazy('Bulk delete'),
+                            include=False,
+                        ),
+                    )
+                ).refine(
                     Prio.table_defaults,
                     fields=declared_bulk_fields,
                 )
@@ -1801,7 +1826,23 @@ class Table(Part, Tag):
 
         self.tbody = self.tbody(_name='tbody', children__text=_Lazy_tbody(self)).bind(parent=self)
         self.container = self.container(_name='container').bind(parent=self)
-        self.outer = self.outer(_name='outer').bind(parent=self)
+
+        self.outer = (
+            self.outer(_name='outer')
+            .refine(
+                children=dict(
+                    h_tag__template=Template('{{ table.h_tag|default_if_none:"" }}'),
+                    query__template=Template('{{ table.query|default_if_none:"" }}'),
+                    actions=dict(
+                        after=LAST if self.actions_below else 'h_tag',
+                        template=Template('{{ table.render_actions|default_if_none:"" }}'),
+                    ),
+                    container__template=Template('{{ table.container|default_if_none:"" }}'),
+                ),
+            )
+            .bind(parent=self)
+        )
+
         self.header = self.header.bind(parent=self)
 
         # needs to be done first because _bind_headers depends on it
@@ -1869,8 +1910,8 @@ class Table(Part, Tag):
         self._bound_members.query = self.query
 
         if self.query is not None:
-            self.sorted_and_filtered_rows = self.query.filter(
-                query=self.query, rows=self.sorted_rows, **self.iommi_evaluate_parameters()
+            self.sorted_and_filtered_rows = self.invoke_callback(
+                self.query.filter, query=self.query, rows=self.sorted_rows
             )
             self.rows = self.sorted_and_filtered_rows
         else:
@@ -2028,9 +2069,9 @@ class Table(Part, Tag):
             rows = self.get_visible_rows()
         else:
             rows = self.sorted_and_filtered_rows
-        preprocessed_rows = self.preprocess_rows(rows=rows, **self.iommi_evaluate_parameters())
+        preprocessed_rows = self.invoke_callback(self.preprocess_rows, rows=rows)
         for i, row in enumerate(preprocessed_rows):
-            row = self.preprocess_row(table=self, row=row)
+            row = self.invoke_callback(self.preprocess_row, row=row)
             assert row is not None, 'preprocess_row must return the row'
             yield self.cells_class(row=row, row_index=i, **self.row.as_dict()).bind(parent=self)
 
@@ -2063,7 +2104,7 @@ class Table(Part, Tag):
         if self.get_request().POST.get('_all_pks_') == '1':
             return 'all'
         else:
-            return [key[len(prefix):] for key in self.get_request().POST if key.startswith(prefix)]
+            return [key[len(prefix) :] for key in self.get_request().POST if key.startswith(prefix)]
 
     def selection(self, prefix='pk_'):
         """Return the selected rows.
